package com.example.service;

import com.azure.data.appconfiguration.ConfigurationClient;
import com.azure.data.appconfiguration.ConfigurationClientBuilder;
import com.azure.data.appconfiguration.models.SettingSelector;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.MethodOrderer.OrderAnnotation;
import org.junit.jupiter.api.Order;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestMethodOrder;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import static org.junit.jupiter.api.Assertions.*;

@TestMethodOrder(OrderAnnotation.class)
class AzureAppConfigServiceTest {

    private static final Logger LOGGER = LoggerFactory.getLogger(AzureAppConfigServiceTest.class);
    private static final String APP_CONFIGURATION_CONNECTION_STRING = System.getenv("APP_CONFIGURATION_CONNECTION_STRING");
    private static final String EXISTING_KEY = "config.message";
    private static final String EXISTING_VALUE = "This is App Config";
    private static final String NEW_KEY = "config.dataSource";
    private static final String NEW_VALUE = "Data Source URL";
    private AzureAppConfigService azureAppConfigService;

    @BeforeEach
    void setUp() {
        ConfigurationClient configurationClient = new ConfigurationClientBuilder().connectionString(APP_CONFIGURATION_CONNECTION_STRING).buildClient();
        azureAppConfigService = new AzureAppConfigService(configurationClient, new SettingSelector());
    }

    @Test
    @Order(1)
    void getValueByKey() {
        String valueByKey = azureAppConfigService.getValueByKey(EXISTING_KEY);
        LOGGER.info(valueByKey);
        assertNotNull(valueByKey);
        assertEquals(EXISTING_VALUE, valueByKey);
    }

    @Test
    @Order(5)
    void getValueByKeyException() {
        assertNull(azureAppConfigService.getValueByKey(null));
    }

    @Test
    @Order(2)
    void writeValueByKey() {
        String writeValueByKey = azureAppConfigService.writeValueByKey(NEW_KEY, NEW_VALUE);
        LOGGER.info(writeValueByKey);
        assertNotNull(writeValueByKey);
        assertEquals(NEW_VALUE, writeValueByKey);
    }

    @Test
    @Order(6)
    void writeValueByKeyException() {
        assertNull(azureAppConfigService.writeValueByKey(null, null));
    }

    @Test
    @Order(4)
    void readAllKeyValue() {
        long count = azureAppConfigService.readAllKeyValue().stream().count();
        assertEquals(3, count);
    }

    @Test
    @Order(3)
    void deleteValueByKey() {
        String valueByKey = azureAppConfigService.deleteValueByKey(NEW_KEY);
        LOGGER.info(valueByKey);
        assertNotNull(valueByKey);
        assertEquals(NEW_VALUE, valueByKey);
    }

    @Test
    @Order(7)
    void deleteValueByKeyException() {
        assertNull(azureAppConfigService.deleteValueByKey(null));
    }
}